##### DISCLAIMER (Please do not edit) ############

# The present routine as been assembled by JF OUSSIN and Mathieu CAZAUNAU (September 2017)
# in the framework of the Eurochamp-2020 Infrastructure activity funded 
# by European Union's Horizon 2020 research and innovation programme 
#under grant agreement No 730997".

# it aims at producing a report that allow two compare the data of two EDF files of exact similar structure (same variables, same order)
#(see description of the format at http:\\http://www.eurochamp.org/Data.aspx). 
# The data are synchronized with respect to a similar event (eg. the time of light on in two different experiments)

# It produces a PDF file located in the original  directory of the first dataset.
# Each category of data will be dispayed on a separate sheet (landscape orientation) . 
# The routine allows to compare a huge number of variable groupe two by two on separated pages "

#The data must be given as in indicated in the model file "exemple.edf". 
# The logo of your institute must be provided in suffiscient resolution 
# in PNG format and should be located in the scriptdir directory under the name "institute_logo.png".

# User input are required only at lines 28, 29 and 34



########################## Empty memory ###########################
rm(list = ls(all = TRUE))

# !!!!!!!!!!!!! WARNING ! USER INPUT REQUESTED !!!!!!!!!!!!!!!!!!!!
# ------ Indicate data and script directories -------------
scriptdir<-"C:/Users/DoussinJF/Documents/JFD/R/Eurochamp-2020/compare2edf/"
indir1<-"C:/Users/DoussinJF/Documents/JFD/R/Eurochamp-2020/compare2edf/"


# !!!!!!!!!!!!! WARNING ! USER INPUT REQUESTED !!!!!!!!!!!!!!!!!!!!
# Name of the first datafile and date/time of the synchronizing event 
filename1<-"exemple2.edf"
start1<- as.POSIXct("2017-11-06 13:35:00" , tz="UTC")

# !!!!!!!!!!!!! WARNING ! USER INPUT REQUESTED !!!!!!!!!!!!!!!!!!!!
# Name of the second datafile and date/time of the synchronizing event 
indir2<-"C:/Users/DoussinJF/Documents/JFD/R/Eurochamp-2020/compare2edf/"
filename2<-"exemple1.edf"
start2<- as.POSIXct("2017-11-03 12:00:00",  tz="UTC")


##################################################################
##                 Read and treat file number 1                 ##
##################################################################

####################### Read all the lines ######################
infile1<-paste(indir1,filename1,sep="")
con <- file(infile1, open = "r")
lines1 <- readLines(con)
close(con)

############# Prepare the file in a standard way ###################
lines1 <- gsub('-9999','0',lines1)
write.table(gsub(',','.',lines1),append=F,quote=F,sep=" ",file='tmp1',col.names=F,row.names=F)

######################## Identify key information ################### 

#----------------------- Find "NUMBER OF COLUMNS=" ------------------
nbcol<-lines1[which(substr(lines1,1,17)=="NUMBER OF COLUMNS")]
#recupration du nombre de colonne
nbcol<-as.integer(unlist(strsplit(nbcol,"="))[2])

#----------------------- Find UNITS(0)-------------------------------
unit<-lines1[which(substr(lines1,1,8)=="UNITS(0)")]

#----------------------- Find initial time -----------------------
date_ini<-unlist((strsplit(unit," ")))[3]
heure_ini<-unlist((strsplit(unit," ")))[4]

#------------------------- Build a title  ---------------------------
Title1<-lines1[which(substr(lines1,1,8)=="CAMPAIGN")]
Title1<-gsub("CAMPAIGN=","",Title1)


colname1<- "Time"
for (icol in seq(2,nbcol)){
  #----------------------- Find SHORT_NAME---------------------------
  if (icol > 10) {
    name<-lines1[which(substr(lines1,1,14)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
  } else {
    name<-lines1[which(substr(lines1,1,13)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
  }
  #----------------------- Find name --------------------------------
  name<-unlist((strsplit(name,"=")))[2]
  
  colname1<- paste(colname1, name, sep=" ")
}

nomcol1=unlist(strsplit(colname1, split=" "))

#---------- Read the data from a line starting with "&" -------------
# (careful all the lines below this point should contain data )
data1<-read.table('tmp1',skip=which(substr(lines1,1,1)=="&"),col.names=nomcol1)

####################### Define the time #####################
date_x1<-as.POSIXct(data1[,1],paste(date_ini,heure_ini,sep=" "),tz="UTC")
date_x1<-date_x1 - start1

#####################create output PDF files ################

for (icol in seq(2,nbcol)){
  #----------------------- Find SHORT_NAME ------------------
  if (icol > 10) {
    name<-lines1[which(substr(lines1,1,14)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
  } else {
    name<-lines1[which(substr(lines1,1,13)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
  }
  #----------------------- Find name ------------------------
  name<-unlist((strsplit(name,"=")))[2]
  
  #----------------------- Find UNITS -----------------------
  if (icol > 10) {
    units<-lines1[which(substr(lines1,1,9)==paste("UNITS(",(icol-1),")",sep=""))]
  } else {
    units<-lines1[which(substr(lines1,1,8)==paste("UNITS(",(icol-1),")",sep=""))]
  }
  
  units<-unlist((strsplit(units,"=")))[2]
  
}

  ##################################################################
  ##                 Read and treat file number 2                 ##
  ##################################################################
  
  ####################### Read all the lines ######################
  infile2<-paste(indir2,filename2,sep="")
  con <- file(infile2, open = "r")
  lines2 <- readLines(con)
  close(con)

  ################ Prepare the file in a standard way ############## 
  lines2 <- gsub('-9999','0',lines2)
  write.table(gsub(',','.',lines2),append=F,quote=F,sep=" ",file='tmp2',col.names=F,row.names=F)
  
  #----------------------- Find "NUMBER OF COLUMNS="----------------
  nbcol<-lines2[which(substr(lines2,1,17)=="NUMBER OF COLUMNS")]
  nbcol<-as.integer(unlist(strsplit(nbcol,"="))[2])
  
  #----------------------- Find UNITS(0) ---------------------------
  unit<-lines2[which(substr(lines2,1,8)=="UNITS(0)")]
  
  #----------------------- Find initial time -----------------------
  date_ini<-unlist((strsplit(unit," ")))[3]
  heure_ini<-unlist((strsplit(unit," ")))[4]
  
  #----------------------- Build a title ---------------------------
  
  Title1<-lines2[which(substr(lines2,1,8)=="CAMPAIGN")]
  Title1<-gsub("CAMPAIGN=","",Title1)
  
  colname2<- "Time"
  

  for (icol in seq(2,nbcol)){
    #----------------------- Find SHORT_NAME -----------------------
    if (icol > 10) {
      name<-lines2[which(substr(lines2,1,14)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
    } else {
      name<-lines2[which(substr(lines2,1,13)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
    }
    #----------------------- Find name -----------------------------
    name<-unlist((strsplit(name,"=")))[2]
    colname2<- paste(colname2, name, sep=" ")
  
  }
  nomcol2=unlist(strsplit(colname1, split=" "))
  
  #---------- Read the data from a line starting with "&" ----------
  # (careful all the lines below this point should contain data )
  data2<-read.table('tmp2',skip=which(substr(lines2,1,1)=="&"),col.names=nomcol2)
  
  ####################### Define the time ##########################
  date_x2<-as.POSIXct(data2[,1],paste(date_ini,heure_ini,sep=" "),tz="UTC")
  date_x2<-date_x2 - start2
  
  ##################### create output PDF files #####################
  date_label <- gsub('/','-',date_ini)
  outfile4<-paste( indir1,filename1,"-vs-", filename2, ".pdf",sep="")
  pdf(outfile4, width = 29.7/2.5, height = 21/2.5) # Open a new pdf file

  ####### Define function for beautiful labels #############
  
  fmt_dcimals <- function(decimals=0){
    function(x) format(x,digits = decimals,scientific = TRUE)
  }
  
fancy_scientific <- function(l) {
    # turn in to character string in scientific notation
    l <- format(l, scientific = TRUE, width=9,digits=2)
    #l <- gsub("0e\\+00","0",l)
    # quote the part before the exponent to keep all the digits
    # l <- gsub("^(.*)e", "'\\1'e", l)
    # turn the 'e+' into plotmath format
    # l <- gsub("e", "%*%10^", l)
    # return this as an expression
    # parse(text=l)
}

##################################################################
##                             PLOT                            ##
##################################################################

library(ggplot2)
library('png')
library('grid')
library(scales)

  ############# Plot the data - one plot over the other ####################
  
  # --------------------- Insert title of the experiment -------------------
  PlotTitle<-name
  
  # -------------------------- Load Logos ----------------------------------
  Logo1<-paste(scriptdir,"institute_logo.png", sep="")
  Logo2<-paste(scriptdir,"eurochamp2020_logo.png", sep="")
  
  img1 <- readPNG(Logo1) 
  Logo_LISA <- rasterGrob(img1, x=unit(0.02, "npc"),y=unit(0.98, "npc"), width = 0.05, height = 0.05, just=c("left", "top")) 
  img2 <- readPNG(Logo2) 
  Logo_EUROCHAMP <- rasterGrob(img2, x=unit(0.88, "npc"),y=unit(0.98, "npc"), width = 0.1, height = 0.07, just=c("left", "top")) 
  
  # --------------------- Make up page title ------------------------------
    MainTitle<-paste( filename1,"\n vs \n", filename2,sep="")
  
  for (icol in seq(2,nbcol)){

    
  #--------------------------- Find UNITS ---------------------------------
        if (icol > 10) {
        units<-lines2[which(substr(lines2,1,9)==paste("UNITS(",(icol-1),")",sep=""))]
      } else {
        units<-lines2[which(substr(lines2,1,8)==paste("UNITS(",(icol-1),")",sep=""))]
        }
    
        units<-unlist((strsplit(units,"=")))[2]

  # --------------------- setting limits of the plot ----------------------
  #x_min<-paste(date_ini,"00:00",sep=" ")
  #x_max<-paste(date_ini,"23:59",sep=" ")
  x_min<-min((date_x1),(date_x2)) 
  x_max<-max((date_x1),(date_x2))
  lims <- as.POSIXct(strptime(c(x_min,x_max), format = "%H:%M", tz="UTC"))    
    y_min<-min((data1[,icol]),(data2[,icol]))
  y_max<- max((data1[,icol]),(data2[,icol]))
  
  # ------------------------- Make the plot -------------------------------
    p1<-ggplot() + 
    geom_line(data=data1, aes(x=date_x1, y=data1[,icol]),colour="green") +
    geom_line(data=data2, aes(x=date_x2, y=data2[,icol]),colour="red") +
    geom_line(x=0, colour="black")+ 
    annotate("text", x=x_max*0.75, y=y_max*0.9, label= filename1,color="green") + 
    annotate("text", x = x_max*0.75, y=y_max*0.8, label = filename2, color="red")+
    xlab("Time \n (seconds since light on)") +
    ylab(paste(colnames(data1[icol]), "\n", units, sep="")) +
    scale_x_continuous(limits =lims) +
    # scale_y_continuous(labels=fancy_scientific)+
    theme(legend.title = element_text(colour="white", size=8), 
          legend.text = element_text(colour="white", size=-0.5*nbcol+15, face="bold"),
          axis.text.x = element_text(colour="grey20",size=-0.3*nbcol+15,angle=0,hjust=.5,vjust=.5,face="plain"),
          axis.text.y = element_text(colour="grey20",size=-0.3*nbcol+15,angle=0,hjust=1,vjust=0,face="plain"),  
          axis.title.x = element_text(colour="grey20",size=-0.5*nbcol+15,angle=0,hjust=.5,vjust=0,face="plain"),
          axis.title.y = element_text(colour="grey20",size=-0.5*nbcol+14,angle=90,hjust=.5,vjust=.5,face="plain"))+
    guides(linetype=guide_legend(title=NULL))
  
  ######################### Produce Report PDF #############################

      pushViewport(viewport(layout = grid.layout(nrow=2, heights = unit(c(.2,1),"null"), ncol=3,widths = unit(c(.5,5,.5), "null"))))
      print(grid.draw(Logo_LISA), vp = viewport(layout.pos.row = 1, layout.pos.col = 3))
      print(grid.draw(Logo_EUROCHAMP), vp = viewport(layout.pos.row = 1, layout.pos.col = 3))
      print(p1, vp = viewport(layout.pos.row = 2, layout.pos.col = 2), just="center") 
      grid.text(MainTitle, x=unit(0.5, "npc"),y=unit(0.50, "npc"), vp = viewport(layout.pos.row = 1, layout.pos.col = 1:3))
  
  #########################################################################"
  grid.newpage()
  
  }
  
  dev.off() # Close the file


